<?php
class UrlLoader
{
    /**
     * @var bool|string
     */
    protected $_content;

    /**
     * @var string
     */
    protected $_error;

    /**
     * @var array
     */
    protected $_header;

    /**
     * UrlLoader constructor.
     *
     * @param string $url
     */
    public function __construct(string $url)
    {
        $rs = curl_init();
        curl_setopt($rs, CURLOPT_URL, $url);
        curl_setopt($rs, CURLOPT_USERAGENT, 'Mozilla/4.0 (compatible; MSIE 6.0; Windows NT 5.1; .NET CLR 1.1.4322)');
        curl_setopt($rs, CURLOPT_HEADER, 1);
        curl_setopt($rs, CURLOPT_RETURNTRANSFER, true);
        curl_setopt($rs, CURLOPT_SSL_VERIFYHOST, false);
        curl_setopt($rs, CURLOPT_SSL_VERIFYPEER, false);
        //curl_setopt($rs, CURLOPT_FILETIME, true);
        $content = curl_exec($rs);
        $this->_error = curl_error($rs);
        $this->_header = curl_getinfo($rs);
        $this->_parseResponse($content);
        @curl_close($rs);

        return $this;
    }

    /**
     * Parsing response data
     *
     * @param string $data
     * @return $this
     */
    protected function _parseResponse(string $data)
    {
        $this->_content = substr($data, $this->getHeader('header_size'));
        $header = substr($data, 0, $this->getHeader('header_size'));
        if (preg_match_all('/([^:\s]+):(.*)/m', $header, $matches) && isset($matches[1])) {
            foreach ($matches[1] as $key => $name) {
                $name = strtolower($name);
                $value = $matches[2][$key] ?? '';
                if (!empty($name)) {
                    $this->_header[$name] = $value;
                }
            }
        }

        return $this;
    }

    /**
     * Returns content
     *
     * @return bool|string
     */
    public function getContent()
    {
        return $this->_content;
    }

    /**
     * Returns error message
     *
     * @return string
     */
    public function getError()
    {
        return $this->_error;
    }

    /**
     * Returns header data
     *
     * @return array|mixed
     */
    public function getHeader(string $name = null)
    {
        return !empty($name) ? ($this->_header[$name] ?? '') : $this->_header;
    }
}