<?php
/**
 * @link http://www.site-search-pro.com
 * @copyright Site Search Pro (tm)

 * @package SearchPro
 * @version 2.0 Gold
**/
/** $Id: rtf2html.class.php,v2.0 2017/09/18 advocat Exp $ */

/**
 * RTF2HTML Class
**/

define('RTF2HTML_ERROR_READFILE',   201);
define('RTF2HTML_ERROR_EMPTY',      202);

class Rtf2Html {
    var $input                  = "";
    var $error                  = 0;
    var $output                 = "";
    
    /** htmlpasver vars */
    var $ptext                  = "";
    var $parbr                  = 0;
    var $par_is_p               = 0;
    var $p_count                = 0;
    
    var $par_html               = "\n";
    
    var $struct                 = array(
    'B'         => 0,
    'I'         => 0,
    'U'         => 0,
    'SUP'       => 0,
    'SAB'       => 0,
    'CAPS'      => 0,
    'QC'        => 0,
    'QR'        => 0,
    'FOOTNOTE'  => 0
    );
    
    var $table_special          = 0;
    var $table_text             = 0;
    var $flag_caps              = 0;
    var $flag_font              = 0;
    var $flag_b                 = 0;
    var $flag_i                 = 0;
    var $flag_u                 = 0;
    var $flag_li                = 0;
    var $flag_ul                = 0;
    var $flag_table             = 0;
    var $flag_td                = 0;
    var $flag_b_r               = 0;
    var $flag_i_r               = 0;
    var $flag_u_r               = 0;
    
    var $fontcenter             = "\n";
    var $fontright              = "\n";
    var $fontleft               = "\n";
    var $fontjustify            = "\n";
    
    var $fontleft0              = "font face='verdana, helvetica ce, arial ce, helvetica, arial' size='2'";
    var $fontfootnote           = "\n";
    
    var $stack                  = array();
    var $stack_count            = 0;
    
    var $footnote_stack         = array();
    var $footnote_count         = 0;
    var $footnote_txt           = '';
    
    var $footnotecolor1         = "<font color='#000000'>";
    var $footnotecolor2         = "</font>";
    
    var $special_footnotes      = 1;
    var $special_footnotes_flag = 0;
    
    function read_file($file)
    {
        $fp = @fopen($file, "r");
        if (!$fp) {
            $this->error = RTF2HTML_ERROR_READFILE;
            return false;
        } else {
            $data = '';
            while (!feof($fp)) $data .= fread($fp, 1024);
            fclose($fp);
            if (!$data) {
                $this->error = RTF2HTML_ERROR_EMPTY;
                return false;
            }
            $this->input = $data;
            $this->parse();
            return true;
        }
    }
    
    function read_data ($data)
    {
        if (!$data) {
            return false;
        } else {
            $this->input = stripslashes($data);
            $this->parse();
            return true;
        }
    }
    
    function get_text () {
        $output = strip_tags($this->output);
        $output = str_replace('&nbsp;', ' ', $output);
        $output = preg_replace("/\s+/si", " ", $output);
        return $output;
    }
    
    function parse ()
    {
        $output = $this->_prep($this->input);
        $output = $this->_correc($output);
        $output = $this->_images($output);
        $output = $this->_convert2html($output);
        $output = $this->_parsehtml($output);
        $output = $this->_htmlcorr($output);
        $output = $this->_small_meta($output);
        $this->output = $output;
    }
    
    function _prep ($input)
    {
        $output = $input;
        $pos = strpos($this->input, "\\fonttbl");
        if (!($pos === false)) {
            $pos2 = strpos($this->input, "\\info", $pos);
            if (!($pos === false)) {
                $output = substr($this->input, 0, $pos).substr($this->input, $pos2, strlen($this->input)-$pos2);
            }
        }
        $pos = strpos($output, "\\par");
        if (!($pos === false)) {
            $pos2=strpos($output, ";}", $pos);
            if (!($pos === false)) {
                $output = substr($output,0,$pos2)."<\02!--BACKSLASHSEMICOLON-->\02".substr($output, $pos2+1, strlen($output)-$pos2-1);
            }
        }
        return $output;
    }
    
    function _correc ($input)
    {
        $pattern = array("(\\\\bliptag[0-9]*)","(\\\\blipuid[^}]*})");
        $replace = array("\\\\pict "," ");
        return preg_replace($pattern, $replace, $input);
    }
    
    function _picturesearch ($input, $xmaxrtf) {
        $rtfimage1 = strpos($input,"\\pict ");
        $rtfimage2 = strpos($input,"\\pict\\");
        $rtfimage3 = strpos($input,"\\pict{");
        
        if ($rtfimage1==0) {
            $rtfimage1=$xmaxrtf;
        }
        if ($rtfimage2==0) {
            $rtfimage2=$xmaxrtf;
        }
        if ($rtfimage3==0) {
            $rtfimage3=$xmaxrtf;
        }
        $rtfimage = min($rtfimage1,$rtfimage2,$rtfimage3);
        if ($rtfimage == $xmaxrtf) {
            $rtfimage = 0;
        }
        return $rtfimage;
    }
    
    function _images ($input)
    {
        $rtfimages  = 0;
        $xmaxrtf    = strlen($input)+1000;
        $rtfimage   = $this->_picturesearch($input, $xmaxrtf);
        
        while ($rtfimage > 0) {
            $rtfimages++;
            $rtfbin     = strpos($input, "\\bin", $rtfimage);
            $rtfbrace   = strpos($input, "}", $rtfimage);
            
            if ($rtfbrace == 0) {
                $rtfbrace=$rtfbin+1;
            }
            if ($rtfbin == 0) {
                $rtfbin=$rtfbrace+1;
            }
            if ($rtfbin < $rtfbrace) {
                $rtfbinspace    = strpos($input, " ", $rtfbin);
                $rtfbinlength   = substr($input, $rtfbin+4, $rtfbinspace-$rtfbin-4);
                $input = substr($input, 0, $rtfimage-1).substr($input, $rtfbinspace+$rtfbinlength+1);
            } else {
                $input = substr($input, 0, $rtfimage-1).substr($input,$rtfbrace);
            }
            $rtfimage = $this->_picturesearch($input, $rtfimages);
        }
        return $input;
    }
    
    function _convert2html ($input)
    {
        $pattern = array("(&)","(>([^\02]))","(<([^\02]))","(\02)","(\\00)","(\\012)","(\\015)","(\\\\<!--CR-->)","(\\\\line([^a-zA-Z0-9]))","(\\^\\^([0-9][0-9]*))","( \\^([0-9][0-9]*))","(<!--CR-->\\^([0-9][0-9]*))","({\\^([0-9][0-9]*))","(\\^\\^(\\\\[^ ]* )([0-9][0-9]*))","(\\^(\\\\[^ ]* )\\^(\\\\[^ ]* )([0-9][0-9]*))","( \\^(\\\\[^ ]* )([0-9][0-9]*))","(<!--CR-->\\^(\\\\[^ ]* )([0-9][0-9]*))","({\\^(\\\\[^ ]* )([0-9][0-9]*))","(\\\\\\\\)","(\\\\{)","(\\\\})","(\\\\;)","(\\\\caps([^a-zA-Z0-9]))","({ })","(\\\\'5C)","(\\\\'7b)","(\\\\'7B)","(\\\\'7d)","(\\\\'7D)","(\\\\'a7)","(\\\\'A7)","(\\\\'e3)","(\\\\'E3)","(\\\\'9c)","(\\\\'9C)","(\\\\'e6)","(\\\\'E6)","(\\\\uc[0-9]*)","(\\\\u[^lL0-9][a-zA-Z0-9]*)","(\\\\'85)","(\205)","(\\\\'ad)","(\255)","(\\\\'AD)","(\\\\'a9)","(\251)","(\\\\'A9)","(\\\\'ae)","(\256)","(\\\\'AE)","(\\\\'99)","(\231)","(\\\\ldblquote )","(\\\\rdblquote )","(\\\\dblquote )","(\\\\lquote )","(\\\\rquote )","(\\\\ldblquote)","(\\\\rdblquote)","(\\\\dblquote)","(\\\\lquote)","(\\\\rquote)","(\\\\~ )","(\\\\~)","(\\\\'a0)","(\240)","(\\\\'A0)","(\\\\'91)","(\221)","(\\\\'82)","(\202)","(\\\\'92)","(\222)","(\\\\'b4)","(\264)","(\\\\'B4)","(\\\\'93)","(\223)","(\\\\'94)","(\224)","(\\\\'ab)","(\253)","(\\\\'bb)","(\273)","(\\\\'AB)","(\253)","(\\\\'BB)","(\273)","(\\\\'84)","(\204)","(\\\\'93)","(\223)","(\\\\emdash)","(\\\\endash)","(\\\\'96)","(\226)","(\\\\'b7)","(\267)","(\\\\'B7)","(\\\\'a4)","(\244)","(\\\\'A4)","(\\\\'95)","(\225)","(\\\\'7c)","(\\\\'7C)","(\\\\'7e)","(\\\\'7E)","(\\\\'7f)","(\\\\'7F)","(\\\\'80)","(\200)","(\\\\'81)","(\201)","(\\\\'83)","(\203)","(\\\\'86)","(\206)","(\\\\'87)","(\207)","(\\\\'88)","(\210)","(\\\\'89)","(\211)","(\\\\'8b)","(\213)","(\\\\'8B)","(\\\\'8c)","(\\\\'8C)","(\\\\'8f)","(\\\\'8F)","(\\\\'90)","(\220)","(\\\\'97)","(\227)","(\\\\'98)","(\230)","(\\\\'9b)","(\233)","(\\\\'9B)","(\\\\'9f)","(\\\\'9F)","(\\\\'a0)","(\240)","(\\\\'A0)","(\\\\'a1)","(\\\\'A1)","(\\\\'a2)","(\\\\'A2)","(\\\\'a3)","(\\\\'A3)","(\\\\'a5)","(\\\\'A5)","(\\\\'a6)","(\246)","(\\\\'A6)","(\\\\'a8)","(\\\\'A8)","(\\\\'aa)","(\\\\'AA)","(\\\\'ac)","(\254)","(\\\\'AC)","(\\\\'af)","(\\\\'AF)","(\\\\'b0)","(\\\\'B0)","(\\\\'b1)","(\261)","(\\\\'B1)","(\\\\'b2)","(\\\\'B2)","(\\\\'b3)","(\\\\'B3)","(\\\\'b5)","(\265)","(\\\\'B5)","(\\\\'b6)","(\266)","(\\\\'B6)","(\\\\'b8)","(\\\\'B8)","(\\\\'b9)","(\\\\'B9)","(\\\\'ba)","(\\\\'BA)","(\\\\'bd)","(\\\\'BD)","(\\\\'bf)","(\\\\'BF)","(\\\\'c2)","(\\\\'C2)","(\\\\'c3)","(\\\\'C3)","(\\\\'c6)","(\\\\'C6)","(\\\\'c7)","(\\\\'C7)","(\\\\'ca)","(\\\\'CA)","(\\\\'cb)","(\\\\'CB)","(\\\\'ce)","(\\\\'CE)","(\\\\'d0)","(\\\\'D0)","(\\\\'d1)","(\\\\'D1)","(\\\\'d7)","(\\\\'D7)","(\\\\'de)","(\\\\'DE)","(\\\\'df)","(\\\\'DF)","(\\\\'e2)","(\\\\'E2)","(\\\\'ea)","(\\\\'EA)","(\\\\'eb)","(\\\\'EB)","(\\\\'ee)","(\\\\'EE)","(\\\\'f0)","(\\\\'F0)","(\\\\'f1)","(\\\\'F1)","(\\\\'f7)","(\\\\'F7)","(\\\\'fe)","(\\\\'FE)","(\\\\'ff)","(\\\\'FF)","(\\\\u193\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u193<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u193<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u193 )","(\\\\u193([^0-9]))","(\\\\'c1)","(\\\\'C1)","()","(\\\\u196\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u196<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u196<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u196 )","(\\\\u196([^0-9]))","(\\\\'c4)","(\\\\'C4)","()","(\\\\u268\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u268<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u268<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u268 )","(\\\\u268([^0-9]))","(\\\\'c8)","(\\\\'C8)","()","(\\\\u270\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u270<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u270<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u270 )","(\\\\u270([^0-9]))","(\\\\'cf)","(\\\\'CF)","()","(\\\\u201\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u201<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u201<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u201 )","(\\\\u201([^0-9]))","(\\\\'c9)","(\\\\'C9)","()","(\\\\u282\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u282<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u282<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u282 )","(\\\\u282([^0-9]))","(\\\\'cc)","(\\\\'CC)","()","(\\\\u205\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u205<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u205<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u205 )","(\\\\u205([^0-9]))","(\\\\'cd)","(\\\\'CD)","()","(\\\\u313\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u313<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u313<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u313 )","(\\\\u313([^0-9]))","(\\\\'c5)","(\\\\'C5)","()","(\\\\u318\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u318<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u318<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u318 )","(\\\\u318([^0-9]))","(\\\\'bc)","(\\\\'BC)","()","(\\\\u327\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u327<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u327<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u327 )","(\\\\u327([^0-9]))","(\\\\'d2)","(\\\\'D2)","()","(\\\\u211\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u211<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u211<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u211 )","(\\\\u211([^0-9]))","(\\\\'d3)","(\\\\'D3)","()","(\\\\u212\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u212<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u212<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u212 )","(\\\\u212([^0-9]))","(\\\\'d4)","(\\\\'D4)","()","(\\\\u214\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u214<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u214<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u214 )","(\\\\u214([^0-9]))","(\\\\'d6)","(\\\\'D6)","()","(\\\\u336\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u336<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u336<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u336 )","(\\\\u336([^0-9]))","(\\\\'d5)","(\\\\'D5)","()","(\\\\u340\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u340<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u340<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u340 )","(\\\\u340([^0-9]))","(\\\\'c0)","(\\\\'C0)","()","(\\\\u344\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u344<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u344<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u344 )","(\\\\u344([^0-9]))","(\\\\'d8)","(\\\\'D8)","()","(\\\\u352\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u352<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u352<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u352 )","(\\\\u352([^0-9]))","(\\\\'8a)","(\\\\'8A)","()","(\\\\u356\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u356<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u356<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u356 )","(\\\\u356([^0-9]))","(\\\\'8d)","(\\\\'8D)","()","(\\\\u218\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u218<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u218<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u218 )","(\\\\u218([^0-9]))","(\\\\'da)","(\\\\'DA)","()","(\\\\u366\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u366<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u366<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u366 )","(\\\\u366([^0-9]))","(\\\\'d9)","(\\\\'D9)","()","(\\\\u220\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u220<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u220<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u220 )","(\\\\u220([^0-9]))","(\\\\'dc)","(\\\\'DC)","()","(\\\\u368\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u368<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u368<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u368 )","(\\\\u368([^0-9]))","(\\\\'db)","(\\\\'DB)","()","(\\\\u221\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u221<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u221<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u221 )","(\\\\u221([^0-9]))","(\\\\'dd)","(\\\\'DD)","()","(\\\\u381\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u381<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u381<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u381 )","(\\\\u381([^0-9]))","(\\\\'8e)","(\\\\'8E)","()","(\\\\u225\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u225<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u225<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u225 )","(\\\\u225([^0-9]))","(\\\\'e1)","(\\\\'E1)","()","(\\\\u228\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u228<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u228<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u228 )","(\\\\u228([^0-9]))","(\\\\'e4)","(\\\\'E4)","()","(\\\\u269\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u269<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u269<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u269 )","(\\\\u269([^0-9]))","(\\\\'e8)","(\\\\'E8)","()","(\\\\u271\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u271<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u271<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u271 )","(\\\\u271([^0-9]))","(\\\\'ef)","(\\\\'EF)","()","(\\\\u233\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u233<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u233<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u233 )","(\\\\u233([^0-9]))","(\\\\'e9)","(\\\\'E9)","()","(\\\\u283\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u283<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u283<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u283 )","(\\\\u283([^0-9]))","(\\\\'ec)","(\\\\'EC)","()","(\\\\u237\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u237<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u237<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u237 )","(\\\\u237([^0-9]))","(\\\\'ed)","(\\\\'ED)","()","(\\\\u314\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u314<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u314<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u314 )","(\\\\u314([^0-9]))","(\\\\'e5)","(\\\\'E5)","()","(\\\\u318\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u318<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u318<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u318 )","(\\\\u318([^0-9]))","(\\\\'be)","(\\\\'BE)","()","(\\\\u328\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u328<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u328<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u328 )","(\\\\u328([^0-9]))","(\\\\'f2)","(\\\\'F2)","()","(\\\\u243\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u243<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u243<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u243 )","(\\\\u243([^0-9]))","(\\\\'f3)","(\\\\'F3)","()","(\\\\u244\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u244<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u244<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u244 )","(\\\\u244([^0-9]))","(\\\\'f4)","(\\\\'F4)","()","(\\\\u246\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u246<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u246<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u246 )","(\\\\u246([^0-9]))","(\\\\'f6)","(\\\\'F6)","()","(\\\\u337\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u337<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u337<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u337 )","(\\\\u337([^0-9]))","(\\\\'f5)","(\\\\'F5)","()","(\\\\u341\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u341<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u341<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u341 )","(\\\\u341([^0-9]))","(\\\\'e0)","(\\\\'E0)","()","(\\\\u345\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u345<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u345<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u345 )","(\\\\u345([^0-9]))","(\\\\'f8)","(\\\\'F8)","()","(\\\\u353\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u353<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u353<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u353 )","(\\\\u353([^0-9]))","(\\\\'9a)","(\\\\'9A)","()","(\\\\u357\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u357<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u357<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u357 )","(\\\\u357([^0-9]))","(\\\\'9d)","(\\\\'9D)","()","(\\\\u250\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u250<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u250<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u250 )","(\\\\u250([^0-9]))","(\\\\'fa)","(\\\\'FA)","()","(\\\\u367\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u367<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u367<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u367 )","(\\\\u367([^0-9]))","(\\\\'f9)","(\\\\'F9)","()","(\\\\u252\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u252<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u252<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u252 )","(\\\\u252([^0-9]))","(\\\\'fc)","(\\\\'FC)","()","(\\\\u369\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u369<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u369<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u369 )","(\\\\u369([^0-9]))","(\\\\'fb)","(\\\\'FB)","()","(\\\\u253\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u253<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u253<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u253 )","(\\\\u253([^0-9]))","(\\\\'fd)","(\\\\'FD)","()","(\\\\u382\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u382<!--CR--><!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u382<!--CR-->\\\\'[0-9a-fA-F][0-9a-fA-F])","(\\\\u382 )","(\\\\u382([^0-9]))","(\\\\'9e)","(\\\\'9E)","()","(\\\\-)","(\\\\'e7)","(\347)","(\\\\'E7)","()","(\\\\tab([^a-zA-Z0-9]))","(\\\\super([^a-zA-Z0-9]))","(\\\\super0([^a-zA-Z0-9]))","(\\\\supernone([^a-zA-Z0-9]))","(\\\\nosupersub([^a-zA-Z0-9]))","(\\\\b  )","(\\\\b0  )","(\\\\bnone  )","(\\\\i  )","(\\\\i0  )","(\\\\inone  )","(\\\\ul  )","(\\\\ul0  )","(\\\\ulnone  )","(\\\\b )","(\\\\b0 )","(\\\\bnone )","(\\\\i )","(\\\\i0 )","(\\\\inone )","(\\\\ul )","(\\\\ul0 )","(\\\\ulnone )","(\\\\b([^a-zA-Z0-9]))","(\\\\b0([^a-zA-Z0-9]))","(\\\\bnone([^a-zA-Z0-9]))","(\\\\i([^a-zA-Z0-9]))","(\\\\i0([^a-zA-Z0-9]))","(\\\\inone([^a-zA-Z0-9]))","(\\\\ul([^a-zA-Z0-9]))","(\\\\ul0([^a-zA-Z0-9]))","(\\\\ulnone([^a-zA-Z0-9]))","(\\\\plain([^a-zA-Z0-9]))","(\\\\sa([0-9][0-9]*))","(\\\\sb([0-9][0-9]*))","(\\\\sa[ ]*([0-9][0-9]*))","(\\\\sb[ ]*([0-9][0-9]*))","(\\\\footnote([^a-zA-Z0-9]))","(\\\\chftn([^a-zA-Z0-9]))","(\\\\par([^a-zA-Z0-9]))","(\\\\pard([^a-zA-Z0-9]))","(\\\\qc([^a-zA-Z0-9]))","(\\\\qr([^a-zA-Z0-9]))","(\\\\ql([^a-zA-Z0-9]))","(\\\\qj([^a-zA-Z0-9]))","(\\\\trowd([^a-zA-Z0-9]))","(\\\\cell([^a-zA-Z0-9]))","(\\\\row })","(\\\\row})","(\\\\row<!--CR--><!--CR-->})","(\\\\row<!--CR-->})","(\\\\row([^a-zA-Z0-9]))","(<td></td>)","({\\\\\\*\\\\docvar[ ]*{[^}]*}{[^}]*}})","(\\\\fldrslt[^}]*)","(\\\\fnil [^;]*)","(\\\\s. [^}]*)","(\\\\f.[0-9]* )","(\\\\leveltext[^}]*)","(\\\\levelnumbers[^}]*)","(\\\\fname[ a-zA-Z0-9]*;}[ a-zA-Z0-9]*;})","({\\\\abilist[^}]*)","({\\\\\\*\\\\listtag0})","({\\\\author[^}]*)","({\\\\operator[^}]*})","({\\\\title[^}]*)","({\\\\\\*\\\\company[^}]*})","(\\\\snext[^}]*)","(\\\\sbasedon[^}]*)","(\\\\noproof[ 0-9]*)","(\\\\fttruetype[^}]*)","(\\\\fcharset[0-9]*[^}]*)","({\\\\s[0-9]*\\\\[^}]*)","(\\\\\\*\\\\[^}]*)","(\\\\pnt[^}{<>]*})","(\\\\u([0-9][0-9]*) )","(\\\\u([0-9][0-9]*))","(\\\\[a-zA-Z0-9][a-zA-Z0-9]*  )","(\\\\[\\*';a-zA-Z0-9\\-\\\\]* )","(\\\\[\\*';a-zA-Z0-9\\-\\\\]*)","(;})","(\.}{ )","(,}{ )","(\?}{ )","(!}{ )","(<!--BACKSLASHSEMICOLON-->}{ )","(})","({)","(})","(> , )","(<!--CR--> <!--CR-->)","(<!--CR--> )","(> <)","( ,)","( \\.\\.)","( \\.)","(<i> )","(<b> )","(<u> )","(<sup> )","(</sup></b></i></u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>\\.)","(</sup></b></i></u>\\.)","(</table><table cellpadding='0' cellspacing='0' border='1' align='center' width='90%'>)","(</table><!--CR--><!--CR--><table cellpadding='0' cellspacing='0' border='1' align='center' width='90%'>)","(</table><!--CR--><table cellpadding='0' cellspacing='0' border='1' align='center' width='90%'>)","(<td></td>)","(<td><!--CR--><!--CR--></td>)","(<td><!--CR--></td>)","(<!--BACKSLASH-->)","(<!--BRACERIGHT-->)","(<!--BRACELEFT-->)","(<!--BACKSLASHSEMICOLON-->)","(<sup></sup>)","(<sup> </sup>)","(<sup>\)</sup>)","(<sup>([^<>/]*)</sup>)","(<sup>)","(</sup>)","(\[[ ]*\])","(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'></u>)","(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'></i>)","(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'></b>)","(<!--CR-->)","(<!--SPACE-->)","((<b>[^<]*)<b>)","((<i>[^<]*)<i>)","((<u>[^<]*)<u>)","( s )","( z )","( k )","( v )","(</p>)","(http://([~a-zA-Z0-9\\._%\\-/:\\?%=+&;]*))","(https://([~a-zA-Z0-9\\._%\\-/:\\?%=+&;]*))","(ftp://([~a-zA-Z0-9\\._%\\-/:\\?%=+\100]*))","(([-a-zA-Z0-9\\._]*)\100([-a-zA-Z0-9\\._]*))");
        $replace = array("&amp<\02!--BACKSLASHSEMICOLON-->\02","&gt<\02!--BACKSLASHSEMICOLON-->\02\\1","&lt<\02!--BACKSLASHSEMICOLON-->\02\\1","","","<!--CR-->","<!--CR-->","<br>","<br>\\1","<a name=\"\\1\"><a href=\"\#xx\\1\"><font color='#000000'>[\\1]</font></a>","<a name=\"xx\\1\">&nbsp<!--BACKSLASHSEMICOLON--><a href=\"\#\\1\"><font color='#000000'>[\\1]</font></a>","<a name=\"xx\\1\">&nbsp<!--BACKSLASHSEMICOLON--><a href=\"\#\\1\"><font color='#000000'>[\\1]</font></a>","<a name=\"xx\\1\">&nbsp<!--BACKSLASHSEMICOLON--><a href=\"\#\\1\"><font color='#000000'>[\\1]</font></a>","<a name=\"\\2\"><a href=\"\#xx\\2\"><font color='#000000'>[\\2]</font></a>\\1 ","<a name=\"\\3\"><a href=\"\#xx\\3\"><font color='#000000'>[\\3]</font></a>\\1 \\2 ","<a name=\"xx\\2\">&nbsp<!--BACKSLASHSEMICOLON--><a href=\"\#\\2\"><font color='#000000'>[\\2]</font></a>\\1 ","<a name=\"xx\\2\">&nbsp<!--BACKSLASHSEMICOLON--><a href=\"\#\\2\"><font color='#000000'>[\\2]</font></a>\\1 ","<a name=\"xx\\2\">&nbsp<!--BACKSLASHSEMICOLON--><a href=\"\#\\2\"><font color='#000000'>[\\2]</font></a>\\1 ","<!--BACKSLASH-->","<!--BRACERIGHT-->","<!--BRACELEFT-->","<!--BACKSLASHSEMICOLON-->","<!--CAPS-->\\1","  ","<!--BACKSLASH-->","<!--BRACERIGHT-->","<!--BRACERIGHT-->","<!--BRACELEFT-->","<!--BRACELEFT-->","<!--PARAGRAPH-->","<!--PARAGRAPH-->","\343","\343","\266","\266","\346","\346","","","...<!--DOTS--> ","...<!--DOTS--> ","...<!--DOTS--> ","...<!--DOTS--> ","...<!--DOTS--> ","(C)<!--COPYRIGHT-->","(C)<!--COPYRIGHT-->","(C)<!--COPYRIGHT-->","(R)<!--REGISTERED-->","(R)<!--REGISTERED-->","(R)<!--REGISTERED-->","(TM)<!--TRADEMARK-->","(TM)<!--TRADEMARK-->","\"<!--LEFTDOUBLEQUOTE-->","\"<!--RIGHTDOUBLEQUOTE-->","\"<!--DOUBLEQUOTE-->","'<!--LEFTQUOTE-->","`<!--RIGHTQUOTE-->","\"<!--LEFTDOUBLEQUOTE-->","\"<!--RIGHTDOUBLEQUOTE-->","\"<!--DOUBLEQUOTE-->","'<!--LEFTQUOTE-->","`<!--RIGHTQUOTE-->","&nbsp<!--BACKSLASHSEMICOLON-->","&nbsp<!--BACKSLASHSEMICOLON-->","&nbsp<!--BACKSLASHSEMICOLON-->","&nbsp<!--BACKSLASHSEMICOLON-->","&nbsp<!--BACKSLASHSEMICOLON-->","'<!--LEFTQUOTE-->","'<!--LEFTQUOTE-->","'<!--LEFTQUOTE-->","'<!--LEFTQUOTE-->","`<!--RIGHTQUOTE-->","`<!--RIGHTQUOTE-->","`<!--RIGHTQUOTE-->","`<!--RIGHTQUOTE-->","`<!--RIGHTQUOTE-->","\"<!--LEFTDOUBLEQUOTE-->","\"<!--LEFTDOUBLEQUOTE-->","\"<!--RIGHTDOUBLEQUOTE-->","\"<!--RIGHTDOUBLEQUOTE-->","&lt<!--BACKSLASHSEMICOLON--><!--LEFTFRENCHQUOTE-->","&lt<!--BACKSLASHSEMICOLON--><!--LEFTFRENCHQUOTE-->","&gt<!--BACKSLASHSEMICOLON--><!--RIGHTFRENCHQUOTE-->","&gt<!--BACKSLASHSEMICOLON--><!--RIGHTFRENCHQUOTE-->","&lt<!--BACKSLASHSEMICOLON--><!--LEFTFRENCHQUOTE-->","&lt<!--BACKSLASHSEMICOLON--><!--LEFTFRENCHQUOTE-->","&gt<!--BACKSLASHSEMICOLON--><!--RIGHTFRENCHQUOTE-->","&gt<!--BACKSLASHSEMICOLON--><!--RIGHTFRENCHQUOTE-->","\"<!--LEFTCROSSOVERQUOTE-->","\"<!--LEFTCROSSOVERQUOTE-->","\"<!--RIGHTCROSSOVERQUOTE-->","\"<!--RIGHTCROSSOVERQUOTE-->","-<!--EMDASH-->","-<!--ENDASH-->","-<!--ENDASH-->","-<!--ENDASH-->","<ul><li>","<ul><li>","<ul><li>","<ul><li>","<ul><li>","<ul><li>","<ul><li>","<ul><li>","\174","\174","\176","\176","\043","\043","Euro<!--EURO-->","Euro<!--EURO-->","?","?","?","?","+<!--DAGGER-->","+<!--DAGGER-->","++<!--DOUBLEDAGGER-->","++<!--DOUBLEDAGGER-->","?","?","promile<!--PROMILE-->","promile<!--PROMILE-->","&lt<!--BACKSLASHSEMICOLON--><!--1FQUOTEL-->","&lt<!--BACKSLASHSEMICOLON--><!--1FQUOTEL-->","&lt<!--BACKSLASHSEMICOLON--><!--1FQUOTEL-->","\246","\246","\254","\254","?","?","-<!--EMDASH-->","-<!--EMDASH-->","?","?","&gt<!--BACKSLASHSEMICOLON--><!--1FQUOTER-->","&gt<!--BACKSLASHSEMICOLON--><!--1FQUOTER-->","&gt<!--BACKSLASHSEMICOLON--><!--1FQUOTER-->","\274","\274","&nbsp<!--BACKSLASHSEMICOLON-->","&nbsp<!--BACKSLASHSEMICOLON-->","&nbsp<!--BACKSLASHSEMICOLON-->","\267","\267","\242","\242","\243","\243","\241","\241","|","|","|","\250","\250","\252","\252","-","-","-","\257","\257","\260","\260","+-<!--PLUSMINUS-->","+-<!--PLUSMINUS-->","+-<!--PLUSMINUS-->","\262","\262","\263","\263","u<!--MICRO-->","u<!--MICRO-->","u<!--MICRO-->","PI<!--PI-->","PI<!--PI-->","PI<!--PI-->","\270","\270","\261","\261","\272","\272","\275","\275","\277","\277","\302","\302","\303","\303","\306","\306","\307","\307","\312","\312","\313","\313","\316","\316","\320","\320","\321","\321","\327","\327","\336","\336","\337","\337","\342","\342","\352","\352","\353","\353","\356","\356","\360","\360","\361","\361","\367","\367","\376","\376","\377","\377","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","\\1","","","","","","","","&nbsp<!--BACKSLASHSEMICOLON-->","&nbsp<!--BACKSLASHSEMICOLON-->&nbsp<!--BACKSLASHSEMICOLON-->&nbsp<!--BACKSLASHSEMICOLON-->&nbsp<!--BACKSLASHSEMICOLON-->&nbsp<!--BACKSLASHSEMICOLON--><!--TAB-->\\1","<sup>\\1","</sup>\\1","</sup>\\1","</sup>\\1","<b><!--SPACE-->","</b><!--SPACE--><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</b><!--SPACE--><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","<i><!--SPACE-->","</i><!--SPACE--><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</i><!--SPACE--><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","<u><!--SPACE-->","</u><!--SPACE--><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</u><!--SPACE--><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","<b>","</b><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</b><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","<i>","</i><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</i><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","<u>","</u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","<b>\\1","</b><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>\\1","</b><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>\\1","<i>\\1","</i><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>\\1","</i><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>\\1","<u>\\1","</u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>\\1","</u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>\\1","</sup></b></i></u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>\\1","<!--SA \\1 -->","<!--SB \\1 -->","<!--SA \\1 -->","<!--SB \\1 -->","<!--FOOTNOTE-->\\1","<!--CHFTN-->\\1","<!--PAR-->\\1","<!--PARD-->\\1","<!--QC-->\\1","<!--QR-->\\1","<!--QL-->\\1","<!--QJ-->\\1","<table cellpadding='0' cellspacing='0' border='1' align='center' width='90%'><td>\\1","</td><td>\\1","</td></table>","</td></table>","</td></table>","</td></table>","</td><tr><td>\\1","","","","","","<!--CR-->","","","","","<!--CR-->","","","","","","","","","","","","","<!--\\1-->","<!--\\1-->","<!--SPACE-->","","","}",".}{  ",",}{  ","?}{  ","!}{  ","<!--BACKSLASHSEMICOLON-->}{  ","</sup></b></i></u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>}","<!--GROUP-->","<!--UNGROUP-->",">, ","<!--SPACE--><!--CR-->","<!--SPACE-->","><",",","  ..",".","<i>","<b>","<u>","<sup>","</sup></b></i></u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>.","</sup></b></i></u>.","<tr>","<tr>","<tr>","","","","\\\\","{","}",";","","",""," [\\1]","","","","</u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</i><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</b><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>",""," ","\\1","\\1","\\1"," s&nbsp;"," z&nbsp;"," k&nbsp;"," v&nbsp;","\n</p>","<a href=\"http://\\1\"><font color='#000000'>\\1</font></a>","<a href=\"https://\\1\"><font color='#000000'>\\1</font></a>","<a href=\"ftp://\\1\"><font color='#000000'>\\1</font></a>","<a href=\"mailto:\\1\100\\2\"><font color='#000000'>\\1\100\\2</font></a>");
        
        $output = preg_replace($pattern, $replace, $input);
        
        do {
            $input = $output;
            $output = preg_replace("(<a href=\"([a-zA-Z]*)://([^&]*)&amp;([^\"]*))", "<a href=\"\\1://\\2\02\\3", $input);
        } while ($input != $output);
        
        $pattern = array("(\02)");
        $replace = array("&");
        
        $input = preg_replace($pattern, $replace, $input);
        return preg_replace($pattern, $replace, $input);
    }
    
    function _parsehtml($input)
    {
        $this->_setparbr(1);
        if (substr_count($input, '<table') != substr_count($input, '!@!</table>')) {
            $this->table_special = 1;
        }
        
        // print $input; OK
        
        $this->output = '';
        $this->_htmlparser($input);
        
        $output = $this->output;
        
        $count = 0;
        while ($count < $this->footnote_count) {
            $this->_newpar($this->ptext.$this->fontfootnote);
            $this->stack[$this->stack_count++] = $this->footnote_stack[$count]['STRUCT'];
            $this->struct['FOOTNOTE'] = 0;
            $this->_scmp();
            $this->_spop();
            $x = $this->footnote_stack[$count]['TEXT'];
            
            $output .= $x;
            $this->struct = $this->footnote_stack[$count]['STRUCT_END'];
            $count++;
        }
        return $output;
    }
    
    function _setparbr($x)
    {
        if ($this->par_is_p) {
            $x = 0;
        }
        $this->parbr = $x;
        if ($this->parbr) {
            $this->ptext= '<br>';
        } else {
            $this->ptext = '<p>';
        }
    }
    
    function _newpar($text)
    {
        if ($this->flag_b) {
            $this->output .= "</b>";
        } if ($this->flag_i) {
            $this->output .= "</i>";
        } if ($this->flag_u) {
            $this->output .= "</u>";
        }
        if (isset($this->struct['SUP']) && $this->struct['SUP']) {
            $this->output .= "</sup>";
        } if ($this->flag_li) {
            $this->output .= "</li>";
        } if ($this->flag_ul) {
            $this->output .= "</ul>";
        }
        
        $this->flag_li = 0;
        $this->flag_ul = 0;
        
        $this->output .= $this->par_html;
        
        if (!$this->parbr) {
            $this->output .= "</p>";
        } elseif ($this->struct['QR'] or $this->struct['QC']) {
            $this->output .= "</p>";
        }
        
        $this->output .= $text;
        
        if ($this->flag_b) {
            $this->output .= "<b>";
        } if ($this->flag_i) {
            $this->output .= "<i>";
        } if ($this->flag_u) {
            $this->output .= "<u>";
        } if (isset($this->struct['SUP']) && $this->struct['SUP']) {
            $this->output .= "<sup>";
        }
    }
    
    function _spush()
    {
        $this->struct['CAPS']   = $this->flag_caps;
        $this->struct['B']      = $this->flag_b;
        $this->struct['I']      = $this->flag_i;
        $this->struct['U']      = $this->flag_u;
        
        $this->stack[$this->stack_count++] = $this->struct;
    }
    
    function _spop()
    {
        if ($this->stack_count==0) {
            return;
        }
        
        $this->struct       = $this->stack[--$this->stack_count];
        $this->flag_caps    = $this->struct['CAPS'];
        $this->flag_b       = $this->struct['B'];
        $this->flag_i       = $this->struct['I'];
        $this->flag_u       = $this->struct['U'];
        
        $this->_setparbr(!$this->struct['SAB']);
    }
    
    function _printflag ($command,$state)
    {
        if ($state) {
            $this->output .= '<'.$command.'>';
        } else {
            $this->output .= '</$'.command.'>';
        }
    }
    
    function _scmp()
    {
        if ($this->stack_count == 0) {
            return;
        }
        $x = $this->stack[$this->stack_count-1];
        
        if ($x['FOOTNOTE'] <> $this->struct['FOOTNOTE']) {
            $this->footnote_stack[$this->footnote_count-1]['TEXT'] = $this->output;
            
            $this->struct['CAPS']       = $this->flag_caps;
            $this->struct['B']          = $this->flag_b;
            $this->struct['I']          = $this->flag_i;
            $this->struct['U']          = $this->flag_u;
            $this->struct['FOOTNOTE']   = 0;
            
            $this->footnote_stack[$this->footnote_count-1]['STRUCT_END'] = $this->struct;

            $this->output = $this->footnote_txt;
            $this->footnote_txt = '';
            $this->_spop();
            $x = $this->stack[$this->stack_count-1];
            $this->struct['FOOTNOTE'] = 0;
        }

        if ($this->flag_b <> $x['B']) {
            $this->_printflag('b',$x['B']);
        }
        if ($this->flag_i <> $x['I']) {
            $this->_printflag('i',$x['I']);
        }
        if ($this->flag_u <> $x['U']) {
            $this->_printflag('u',$x['U']);
        }
        if ($x['SUP'] <> $this->struct['SUP']) {
            $this->_printflag('sup',$x['SUP']);
        }

        if (($x['QC'] <> $this->struct['QC']) || ($this->struct['QR'] <> $x['QR'])) {
            /* change state */
            if ($x['QC']) {
                $this->_newpar('<p align="center">'.$this->fontcenter);
            } else if ($x['QR']) {
                $this->_newpar('<p align="right">'.$this->fontright);
            } else {
                $this->_newpar($this->ptext.$this->fontleft);
            }
        }
        
    }
    
    function _htmlparser_text ($htmltext)
    {
        if ($this->flag_caps) {
            $htmltext = $this->_xcupper($htmltext);
        }
        if ($this->table_special && $this->flag_table) {
            if (strlen(trim($htmltext)) > 40) {
                if ($this->flag_font) {
                    $this->output .= "</font>";
                    $this->flag_font = 0;
                    $this->flag_caps = 0;
                }
                if ($this->flag_b) {
                    $this->output .= "</b>";
                    $this->flag_b = 0;
                    $this->flag_b_r = 0;
                }
                if ($this->flag_i) {
                    $this->output .= "</i>";
                    $this->flag_i = 0;
                    $this->flag_i_r = 0;
                }
                if ($this->flag_u) {
                    $this->output .= "</u>";
                    $this->flag_u = 0;
                    $this->flag_u_r = 0;
                }
                if ($this->flag_td) {
                    $this->output .= "</td>";
                    $this->flag_td = 0;
                }
                $this->output .= "</table>";
                $this->flag_table = 0;

                $this->output .= "</p>".$this->ptext.$this->fontleft."\n";
                $this->output .= $htmltext;
                
                return;
            }
        }
        
        /** table without <td> */
        if (!$this->flag_td && $this->flag_table) {
            $this->output .= "<td>".$this->fontleft."\n";
            $this->flag_td = 1;
            $this->flag_font = 1;
        }
        /** without font */
        if (!$this->flag_font) {
            $this->output .= $this->fontleft."\n";
            $this->flag_font = 1;
            $this->flag_caps = 0;
        }
        /** td in the table */
        if ($this->flag_td)	{
            if ($this->flag_b_r && !$this->flag_b) {
                $this->output .= "<b>";
                $this->flag_b = 1;
            }
            if ($this->flag_i_r && !$this->flag_i) {
                $this->output .= "<i>";
                $this->flag_i = 1;
            }
            if ($this->flag_u_r && !$this->flag_u) {
                $this->output .= "<u>";
                $this->flag_u = 1;
            }
        }
        $this->output .= $htmltext;
    }

    function _htmlparser_comment($htmltext)
    {
        if ((!strncmp($htmltext, "SA ", 3)) or (!strncmp($htmltext, "SB ", 3))) {
            strtok($htmltext, " ");
            $x = strtok(" ");
            $this->struct['SAB'] = $x;
            $this->_setparbr(!$x);
            return;
        }
        
        switch ($htmltext) {
            case "CAPS":
                $this->flag_caps=1;
                break;
            case "ENDCAPS":
                $this->flag_caps=0;
                break;
            case "GROUP":
                $this->_spush();
                break;
            case "UNGROUP":
                $this->_scmp();
                $this->_spop();
                break;
            case "PARD":
                /** reset */
                if (($this->struct['QC']) or ($this->struct['QR'])) {
                    $this->_newpar($this->ptext.$this->fontleft);
                }
                $this->struct['QC'] = 0;
                $this->struct['QR'] = 0;
                $this->struct['SAB'] = 0;
                $this->_setparbr(1);
                break;
            case "PAR":
                if ($this->struct['QC']) {
                    $this->_newpar('<p align="center">'.$this->fontcenter);
                } else if ($this->struct['QR']) {
                    $this->_newpar('<p align="right">'.$this->fontright);
                } else {
                    $this->_newpar($this->ptext.$this->fontleft);
                }
                break;
            case "QC":
                if (!$this->struct['QC']) {
                    $this->_newpar('<p align="center">'.$this->fontcenter);
                    $this->struct['QC'] = 1;
                    $this->struct['QR'] = 0;
                }
                break;
            case "QR":
                if (!$this->struct['QR']) {
                    $this->_newpar('<p align="right">'.$this->fontright);
                    $this->struct['QC'] = 0;
                    $this->struct['QR'] = 1;
                }
                break;
            case "QL":
            case "QJ":
                if ($this->struct['QC'] or $this->struct['QR']) {
                    $this->_newpar($this->ptext.$this->fontleft);
                    $this->struct['QC'] = 0;
                    $this->struct['QR'] = 0;
                }
                break;
            case "FOOTNOTE":
                if (!$this->struct['FOOTNOTE']) {
                    $this->_spush();
                    $this->footnote_stack[$this->footnote_count++]['STRUCT'] = $this->struct;
                    $this->struct['FOOTNOTE'] = 1;
                    $this->footnote_txt = $this->output;
                    $this->output = '';
                }
                break;
            case "CHFTN":
                if ($this->struct['FOOTNOTE']) {
                    $fname='<a name="'.$this->footnote_count.'"><a href="#xx'.$this->footnote_count.'">'.$this->footnotecolor1.'['.$this->footnote_count.']'.$this->footnotecolor2.'</a>';
                    $this->output .= " ".$fname." ";
                } else {
                    $this->footnote_count++;
                    $ftarget = '<a name="xx'.$this->footnote_count.'">&nbsp;<a href="#'.$this->footnote_count.'">'.$this->footnotecolor1.'['.$this->footnote_count.']'.$this->footnotecolor2.'</a>';
                    $this->output .= " ".$ftarget." ";
                    $this->footnote_count--;
                }
                break;
            default:
                $this->output .= "<!--".$htmltext."-->";
        }
    }

    function _htmlparser_command($htmltext)
    {
        $htmltext0 = strtolower($htmltext);
        switch($htmltext0) {
            case "li":
            $this->flag_li = 1;
            $this->output .= "<li>";
            break;

            case "/li":
            $this->flag_li = 0;
            $this->output .= "</li>";
            break;

            case "ul":
            $this->flag_ul = 1;
            $this->output .= "<ul>";
            break;

            case "/ul":
            $this->flag_ul = 0;
            $this->output .= "</ul>";
            break;

            case "sup":
            $this->output .= "<sup>";
            $this->struct['SUP'] = 1;
            break;

            case "/sup":
            $this->output .= "</sup>";
            $this->struct['SUP'] = 0;
            break;

            case "/font":
            $this->output .= "</font>";
            $this->flag_font = 0;
            $this->flag_caps = 0;
            break;

            case "b":
            if (!$this->flag_b) {
                $this->output .= "<b>";
                $this->flag_b = 1;
                $this->flag_b_r= 1;
            }
            break;

            case "i":
            if (!$this->flag_i) {
                $this->output .= "<i>";
                $this->flag_i = 1;
                $this->flag_i_r = 1;
            }
            break;

            case "u":
            if (!$this->flag_u) {
                $this->output .= "<u>";
                $this->flag_u = 1;
                $this->flag_u_r = 1;
            }
            break;

            case "/b":
            $this->flag_b_r = 0;
            if ($this->flag_b) {
                $this->output .= "</b>";
                $this->flag_b = 0;
                $this->flag_b_r = 0;
            }
            break;


            case "/i":
            $this->flag_i_r=0;
            if ($this->flag_i) {
                $this->output .= "</i>";
                $this->flag_i = 0;
                $this->flag_i_r = 0;
            }
            break;

            case "/u":
            $this->flag_u_r=0;
            if ($this->flag_u) {
                $this->output .= "</u>";
                $this->flag_u = 0;
                $this->flag_u_r = 0;
            }
            break;

            default:
            if ($this->special_footnotes && !$this->special_footnotes_flag) {
                if (!strncmp($htmltext0, "a href=\"#xx", 11)) {
                    $this->special_footnotes_flag = 1;
                    $this->fontleft = $this->fontfootnote;
                    $this->output .= "<".$htmltext.">";
                    return;
                }
            }

            if (!strncmp($htmltext0,"font",4)) {
                if ($this->special_footnotes_flag) {
                    $this->output .= $this->fontleft;
                    $this->flag_font = 1;
                    $this->flag_caps = 0;
                    return;
                }

                $this->flag_font=1;
                $this->flag_caps=0;
                $this->output .= "<".$htmltext.">";
                return;
            } elseif (!strncmp($htmltext0,"table",5)) {
                $this->table_text = 0;
                if ($this->flag_table && $this->table_special) {
                    $this->output .= "<tr>";
                } else {
                    $this->output .= "<".$htmltext.">";
                    $this->flag_table = 1;
                }
                return;
            } elseif (!strncmp($htmltext0,"/table",6)) {
                $this->table_text = 0;
                if (!$this->table_special) {
                    $this->output .= "<".$htmltext.">";
                    $this->flag_table = 0;
                } elseif ($this->flag_table) {
                    $this->output .= "<tr>";
                }
                return;
            } elseif (!strncmp($htmltext0,"td",2)) {
                $this->p_count = 0;
                $this->table_text = 0;
                if ($this->flag_td) {
                    if ($this->flag_font) {
                        $this->output .= "</font>";
                    }
                    $this->output .= "</td>";
                }
                $this->flag_td      = 1;
                $this->flag_font    = 0;
                $this->flag_b       = 0;
                $this->flag_i       = 0;
                $this->flag_u       = 0;

                $this->output .= "<".$htmltext.">";
                return;
            } elseif (!strncmp($htmltext0,"/td",3)) {
                $this->p_count = 0;
                if ($this->flag_b) {
                    $this->output .= "</b>";
                    $this->flag_b = 0;
                }
                if ($this->flag_i) {
                    $this->output .= "</i>";
                    $this->flag_i = 0;
                }
                if ($this->flag_u) {
                    $this->output .= "</u>";
                    $this->flag_u = 0;
                }
                if ($this->flag_font) {
                    $this->output .= "</font>";
                }
                $this->table_text   = 0;
                $this->flag_td      = 0;
                $this->flag_font    = 0;
                $this->output .= "<".$htmltext.">";
                return;
            }
            
            $this->output .= "<".$htmltext.">";
        }
    }

    function _htmlparser($i)
    {
        $off    = 0;
        $lng    = strlen($i);
        $maxx   = $lng+1;
        
        do {
            if ($off >= $maxx) {
                return;
            }
            
            $comm_pos = strpos($i, "<!--", $off);
            if ($comm_pos === false) {
                $comm_pos = $maxx;
            }
            $cmd_pos = strpos($i, "<", $off);
            if ($cmd_pos === false) {
                $cmd_pos = $maxx;
            }
            $off++;

            if (($cmd_pos > $off) && ($comm_pos > $off)) {
                $end_text = strpos($i, "<", $off);
                if ($end_text === false) {
                    $end_text=$maxx;
                }
                
                $htmltext = substr($i,$off,$end_text-$off);
                if ($htmltext) {
                    $this->_htmlparser_text($htmltext);
                }
                $off = $end_text;
            } elseif ($comm_pos <= $off) {
                $end_text = strpos($i,"-->",$off);
                if ($end_text === false) {
                    $end_text=$maxx;
                }
                $htmltext = substr($i, $comm_pos+4, $end_text-$comm_pos-4);
                $htmltext = trim($htmltext);
                
                if ($htmltext) {
                    $this->_htmlparser_comment($htmltext);
                }
                $off = $end_text + 2;
            } elseif ($cmd_pos <= $off) {
                $end_text = strpos($i, ">", $off);
                if ($end_text === false) {
                    $end_text = $maxx;
                }
                
                $htmltext = substr($i, $cmd_pos+1, $end_text-$cmd_pos-1);
                $htmltext = trim($htmltext);
                if ($htmltext) {
                    $this->_htmlparser_command($htmltext);
                }
                $off = $end_text;
            }
        } while (1);
    }
    
    function _xcupper ($xinput)
    {
        $pattern = array("(a)","()","()","(b)","(c)","()","(d)","()","(e)","()","()","()","(f)","(g)","(h)","(i)","()","(j)","(k)","(l)","()","()","(m)","(n)","()","()","(o)","()","()","()","(p)","(q)","(r)","()","()","(s)","()","(t)","()","(u)","()","()","()","(v)","(w)","(x)","(y)","()","(z)","()");
        $replace = array("A","","","B","C","","D","","E","","","","F","G","H","I","","J","K","L","","","M","N","","","O","","","","P","Q","R","","","S","","T","","U","","","","V","W","X","Y","","Z","");
        return preg_replace($pattern, $replace, $xinput);
    }
    
    function _htmlcorr($input)
    {
        $pattern = array("(\[[ ]*\])","((<b>[^<]*)<b>)","((<i>[^<]*)<i>)","((<u>[^<]*)<u>)","(<b>(<[^>]*>)</b>)","(<i>(<[^>]*>)</i>)","(<u>(<[^>]*>)</u>)","(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'></u>)","(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'></i>)","(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'></b>)","(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='3'></u>)","(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='3'></i>)","(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='3'></b>)","(<font[^>]*>\n</font>)");
        $replace = array("","\\1","\\1","\\1","\\1","\\1","\\1","</u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</i><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</b><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>","</u><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='3'>","</i><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='3'>","</b><font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='3'>","");
        $output  = preg_replace($pattern, $replace, $input);
        
        do {
            $input  = $output;
            $output = preg_replace("(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>[\n ]*<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>)","<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='2'>", $input);
        } while ($input != $output);
        
        do {
            $input  = $output;
            $output = preg_replace("(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='3'>[\n ]*<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='3'>)","<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='3'>", $input);
        } while ($input!=$output);
        
        do {
            $input  = $output;
            $output = preg_replace("(<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='1'>[\n ]*<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='1'>)","<font face='Verdana, Helvetica CE, Arial CE, Helvetica, Arial' size='1'>",$input);
        } while ($input != $output);
        return $output;
    }
    
    function _small_meta ($input)
    {
        $pattern = array("(&AMP;)","(&GT;)","(&LT;)","(&NBSP;)");
        $replace = array("&amp;","&gt;","&lt;","&nbsp;");
        return preg_replace($pattern, $replace, $input);
    }
}

?>